#include "sharktorrentframe.h"

BEGIN_EVENT_TABLE( sharkTorrentFrame, wxFrame )
	EVT_MENU( Menu_File_Quit, sharkTorrentFrame::OnQuit )
	EVT_MENU( Menu_File_Open_Torrent, sharkTorrentFrame::OnOpen )
	EVT_MENU( Menu_Settings_Configure, sharkTorrentFrame::openSettings )
	EVT_MENU( Menu_Help_About, sharkTorrentFrame::OnAbout )
	EVT_TIMER(MAIN_TIMER_ID, sharkTorrentFrame::OnTimer)
	EVT_CLOSE(sharkTorrentFrame::OnClose)
END_EVENT_TABLE()


sharkTorrentFrame::sharkTorrentFrame( const wxString& title, const wxPoint& poss, const wxSize& size )
	: wxFrame((wxFrame *)NULL, -1, title, poss, size ), m_timer(this, MAIN_TIMER_ID)
{
	m_lastOnTimer = 0;
	m_hashmapcount =0;
	stFileDropTarget *fdt = new stFileDropTarget() ;
	SetDropTarget(fdt);
	
	m_lastOpenDir = _T("");
	

	SetIcon(wxIcon(st4_xpm));
	m_session = new libtorrent::session(fingerprint("ST", 0, 1, 0, 0));

	m_tbi = new stTaskBarIcon();
		m_tbi->SetIcon(wxIcon(st4_xpm), wxT("sharkTorrent"));
	
	m_timer.Start(2000);

	// TODO: rewrite whole config managment
	wxFileInputStream fis( wxStandardPaths::Get().GetUserDataDir() + _T("/sharktorrent.conf"));
	wxFileConfig *pConfig = new wxFileConfig( (wxInputStream &)fis );
	
	long port = pConfig->Read(_T("/Conn/port"), 6881l);
	long port_end = pConfig->Read(_T("/Conn/port_end"), 6889l);
	
	if(port < 1024 || port > 65535) port = 6881;
	if(port_end < 1024 || port_end > 65535) port_end = 6890;
	
	// this is almost good for linux and win, but probably not for mac ..
	wxString _dldir = wxFileName::GetHomeDir() + _T("/Desktop/sharktorrent/");
	wxString _fdl  = pConfig->Read(_T("/Dirs/fdl"), _dldir);
	_fdl.Replace(_T("\\"), _T("/"));
	if( !wxFileName::DirExists( _fdl )) {	
		bool b = wxFileName::Mkdir(_fdl); 
		if(!b) wxLogMessage(_T("failed to create directory ") + _fdl);
	}
	
	m_session->listen_on(std::make_pair(port, port_end));
	
//	m_session->start_dht();
	
	GenerateMenusAndStatusBar();

	m_splitter = new wxSplitterWindow(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER);
	m_mlistctrl = new stTorrentsListCtrl(m_splitter, TORRENTS_LISTVIEW, wxDefaultPosition, wxDefaultSize, wxLC_REPORT );
	m_mig = new stTorrentMetainfoGui(m_splitter, 101, wxTAB_TRAVERSAL | wxNO_BORDER);
	m_splitter->SplitHorizontally(m_mlistctrl, m_mig, 400);

	ApplySettings();
	
}

void sharkTorrentFrame::FirstInit() {
	wxArrayString *_was1 = new wxArrayString();
	wxDir::GetAllFiles(wxStandardPaths::Get().GetUserDataDir() + _T("/torrents/"), _was1);
	OpenTorrentFiles( *_was1, true);
	
		
	stIPCServer *_dde = new stIPCServer;

	if( !_dde->Create(_T("/tmp/sharktorrent.ipc")) ) {
		std::cout << "failed to create DDE\n";
	} else {
		std::cout << "created DDE\n";
	} 
	if(wxGetApp().argc >=2) {
		//wxString &argvs = wxString::FromAscii(wxGetApp().argv[1]);
		OpenTorrentFile( wxString(wxGetApp().argv[1]) );
	} 
}

void sharkTorrentFrame::GenerateMenusAndStatusBar() {
	wxMenu *menuFile = new wxMenu;
	
	menuFile->Append( Menu_File_Open_Torrent, wxT( "&Open Torrent...\tCtrl-O" ) );
	menuFile->AppendSeparator();
	menuFile->Append( Menu_File_Quit, wxT( "E&xit\tCtrl-Q" ) );
	
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuFile, wxT( "&File" ) );

	// Settings
	menuFile = new wxMenu;
	menuFile->Append( Menu_Settings_Configure, wxT( "&Preferences\tCtrl-P" ) );
	
	menuBar->Append( menuFile, wxT( "&Edit") );

	// HELP
	menuFile = new wxMenu;
	menuFile->Append( Menu_Help_About, wxT( "&About\tF1" ) );
	
	menuBar->Append( menuFile, wxT( "&Help" ) );

	SetMenuBar( menuBar );
	CreateStatusBar();
	SetStatusText( wxT( "Welcome to sharkTorrent" ) );
}

void sharkTorrentFrame::DoQuit() {
	delete m_tbi;
	m_tbi = NULL;
	
	for (stMapHandles::iterator i = m_handles.begin(); i != m_handles.end(); ++i)
	{
		torrent_handle  h = (i->second).GetHandle();
		if (!h.is_valid() || !h.has_metadata()) continue;
		
		h.pause();

		/*
		entry data = h.write_resume_data();
		std::stringstream s;
		s << h.get_torrent_info().name() << ".fastresume";
		boost::filesystem::ofstream out(h.save_path() / s.str(), std::ios_base::binary);
		out.unsetf(std::ios_base::skipws);
		bencode(std::ostream_iterator<char>(out), data);
		*/
		m_session->remove_torrent(h);
	}
	Destroy();
}

void sharkTorrentFrame::OnQuit( wxCommandEvent& WXUNUSED( event ) )
{
	this->DoQuit();
}

void sharkTorrentFrame::OnClose( wxCloseEvent& WXUNUSED( event ) )
{
	Show(false);
}


void sharkTorrentFrame::OnAbout( wxCommandEvent& WXUNUSED( event ) )
{
	wxString _ts = 
			wxString::Format(
			_T("sharkTorrent version "ST_VERSION" .\n"
				"Keep in mind that this is development release...\n\n"
				"Author: sfinks@users.sourceforge.net\n"
				"LICENSE: GNU General Public License\n\n"
				"Uses:\n"
				"libtorrent - http://www.rasterbar.com/products/libtorrent/ \n"
				"wxwidgets - http://www.wxwidgets.org/ \n"
			  	"boost - http://boost.org/\n"
				"asio\n\n"
				"Also, thanks for debian, linux, kdevelop, gcc, mingw, sourceforge.\n\n"
				"Homepage: http://sharktorrent.com/\n"
			  	)
			);
	wxMessageBox(
			_ts,
			_T("About sharkTorrent"), wxOK
			);
}

void sharkTorrentFrame::OpenTorrentFile(wxString s) {
	wxArrayString _was;
	std::cout << "\ns=====" << s.mb_str(wxConvUTF8) << "\n";
	wxString s2 = s;
	_was.Add(s2);
	OpenTorrentFiles(_was, false);
}

void sharkTorrentFrame::OpenTorrentFiles(wxArrayString was, bool bootup ) {
	wxString _ts1;
	wxString _ts2_fn;
	for(int ii = 0 ; ii < was.GetCount() ; ii++) {
		_ts1 = was[ii];
		wxFileName _fn(_ts1);
		try
		{
			std::ifstream in(_ts1.mb_str(wxConvUTF8), std::ios_base::binary);
			in.unsetf(std::ios_base::skipws);
			entry e = bdecode(std::istream_iterator<char>(in), std::istream_iterator<char>());
		
			// TODO: again, config managment .... rewrite.. and .. stuff
			wxFileInputStream fis( wxStandardPaths::Get().GetUserDataDir() + _T("/sharktorrent.conf"));
			wxFileConfig *pConfig = new wxFileConfig( (wxInputStream &)fis );
			wxString fdl  = pConfig->Read(_T("/Dirs/fdl"), wxFileName::GetHomeDir()  + _T("/Desktop/sharktorrent/"));
			// replace windows style backslashes with forward slashes
			fdl.Replace(_T("\\"), _T("/"));
	
			boost::filesystem::path ssave_path( fdl.mb_str(wxConvUTF8));
			torrent_handle h = m_session->add_torrent(torrent_info(e), ssave_path) ;
			
			long maxdl = pConfig->Read( _T("/Conn/maxdl"), 0l);
			if( GetNotSeedNotPausedCount() >= maxdl && maxdl>0) h.pause();
			//std::cout << "dcnt=" << GetNotSeedNotPausedCount() << "\n";
			m_handles[m_hashmapcount++].SetHandle(h);

			if(bootup == false ) {
				int _i = 1;
				wxString _ts2 = wxStandardPaths::Get().GetUserDataDir() + _T("/torrents/") + wxString::Format(_T("%d.torrent"), _i);
				for( ; wxFileExists(_ts2); _i++) {
					_ts2 = wxStandardPaths::Get().GetUserDataDir() + _T("/torrents/") + wxString::Format(_T("%d.torrent"), _i);
				}
	
				if( !wxCopyFile( _ts1, _ts2 , false)) {
					wxLogMessage(_T("copy failed for ")+_ts1+_T(" -> ")+_ts2);
				} else {
					m_handles[m_hashmapcount-1].SetStr(_ts2);
				}
			} else {
				m_handles[m_hashmapcount-1].SetStr(_ts1);
			} 
		} catch (std::exception& e)
		{
			wxLogMessage( wxString::FromAscii(e.what()) );
		}
//		std::cout << "end of for\n";
	} //for
	
	updateLists();
}

void sharkTorrentFrame::OnOpen( wxCommandEvent& WXUNUSED( event ) )
{
	wxFileDialog dia(this, _T("Choose A File"), m_lastOpenDir, _T(""), _T("Torrent Files (*.torrent)|*.torrent|All files (*.*)|*.*"), \
			 wxMULTIPLE|wxFILE_MUST_EXIST|wxOPEN   );

	int f_ret =dia.ShowModal();

	if(f_ret == wxID_OK) {
		m_lastOpenDir = dia.GetDirectory();
	} else {
		return;
	}
	
	wxArrayString f_was, f_was2;;
	wxString f_ts1;
	
	dia.GetFilenames(f_was);
	
	for(int _ii =0 ; _ii < f_was.GetCount() ; _ii++) {
		// this is not needed acctually, because later \\ gets replaced with / anyway..
		f_ts1 = (dia.GetDirectory() + _("/") +  f_was[_ii] );
		f_ts1.Replace(_T("\\"), _T("/"));
		f_was2.Add(f_ts1);

	}

	OpenTorrentFiles(f_was2);
}

void sharkTorrentFrame::OnTimer(wxTimerEvent& event)
{	
	wxLongLong _ca = wxGetLocalTimeMillis();
	if(m_lastOnTimer > 0 && (_ca - m_lastOnTimer)>1000) {
		if(IsShown()) {
			updateLists();
		} else {

		} 
		wxLongLong _ce = wxGetLocalTimeMillis();
		std::cout << "OnTimer from begin to end,  ms= :\t " << (_ce - _ca).ToLong() << "\n";
	} else {
		std::cout << "skipping OnTimer\n";
	} 
	m_lastOnTimer = _ca;
}

void sharkTorrentFrame::updateLists()
{
		m_mlistctrl->updateList( m_handles );
		int f_selectedRowItemData = m_mlistctrl->GetSelectedItemData();
				
		if(f_selectedRowItemData>= 0 && m_mlistctrl->GetSelectedItemCount()==1) 
		{
			torrent_handle h = m_handles[ f_selectedRowItemData ].GetHandle();
			m_mig->updatePeersGui( h );
			m_mig->updateFilesGui( h );
		}
		
		if(m_mlistctrl->GetSelectedItemCount()>1) 
		{
			m_mig->clearAll();
		}
}

/*!
    \fn sharkTorrentFrame::openSettings()
 */
void sharkTorrentFrame::openSettings(wxCommandEvent& WXUNUSED( event ))
{
	wxFileInputStream fis( wxStandardPaths::Get().GetUserDataDir() + _T("/sharktorrent.conf"));
	wxFileConfig *pConfig = new wxFileConfig( (wxInputStream &)fis );
	//long maxdl = pConfig->Read( _T("/Conn/maxdl"), 0l);
	
	long maxdl = pConfig->Read( _T("/Conn/maxdl"), 0l);
	long maxul = pConfig->Read( _T("/Conn/maxul"), 10l);
	long maxseeds = pConfig->Read(_T("/Conn/maxseeds"), 0l);
	long maxconn = pConfig->Read(_T("/Conn/maxconn"), 120l);
	long maxulspeed = pConfig->Read(_T("/Conn/maxulspeed"), 0l);
	long maxdlspeed = pConfig->Read(_T("/Conn/maxdlspeed"), 0l);
	long port = pConfig->Read(_T("/Conn/port"), 6881l);
	long port_end = pConfig->Read(_T("/Conn/port_end"), 6889l);
	long utpport = pConfig->Read(_T("/Conn/utpport"), 4444l);
	//pConfig->SetPath(_T("/Dirs"));
	wxString fdl  = pConfig->Read(_T("/Dirs/fdl"), wxFileName::GetHomeDir()   + _T("/Desktop/sharktorrent/"));
	fdl.Replace(_T("\\"), _T("/"));
	//wxString fdl(_T("./"));

	stConfigDialog *f_dialog = new stConfigDialog(this);
	f_dialog->SetData(maxdl, maxul, maxseeds, maxconn, maxulspeed, maxdlspeed, port, port_end, utpport, fdl);
		
	int f_ret = f_dialog->ShowModal();
	if(f_ret == wxID_OK) {
	} else {
		return;
	}

	long ldl=0, lul=1, lseeds=0, lconn=0, lulspeed=0, ldlspeed=0, lport=0, lport_end=0;//, lutp=0;

	f_dialog->m_maxdl->GetValue().ToLong( &ldl);
	f_dialog->m_maxul->GetValue().ToLong( &lul);
	f_dialog->m_maxseeds->GetValue().ToLong( &lseeds);
	f_dialog->m_maxconn->GetValue().ToLong( &lconn);
	f_dialog->m_maxulspeed->GetValue().ToLong( &lulspeed);
	f_dialog->m_maxdlspeed->GetValue().ToLong( &ldlspeed);
	f_dialog->m_port->GetValue().ToLong( &lport);
	f_dialog->m_port_end->GetValue().ToLong( &lport_end);
//	f_dialog->m_utp->GetValue().ToLong( &lutp);

	pConfig->Write(_T("/Conn/maxdl"), ldl );
	pConfig->Write(_T("/Conn/maxul"), lul );
	pConfig->Write(_T("/Conn/maxseeds"), lseeds );
	pConfig->Write(_T("/Conn/maxconn"), lconn );
	pConfig->Write(_T("/Conn/maxulspeed"), lulspeed );
	pConfig->Write(_T("/Conn/maxdlspeed"), ldlspeed );
	pConfig->Write(_T("/Conn/port"), lport );
	pConfig->Write(_T("/Conn/port_end"), lport_end );
//	pConfig->Write(_T("/Conn/utpport"), lutp );
	pConfig->Write(_T("/Dirs/fdl"), f_dialog->m_fdl->GetValue() );
	
	//pConfig->Flush();

	wxFileOutputStream fos( wxStandardPaths::Get().GetUserDataDir() + _T("/sharktorrent.conf"));
	pConfig->Save(fos);
	
	ApplySettings();
}

void sharkTorrentFrame::ApplySettings() {
	// TODO: rewrite this config managment ....
	wxFileInputStream fis( wxStandardPaths::Get().GetUserDataDir() + _T("/sharktorrent.conf"));
	wxFileConfig *pConfig = new wxFileConfig( (wxInputStream &)fis );

//	long maxdl = pConfig->Read( _T("/Conn/maxdl"), 0l);
	long maxul = pConfig->Read( _T("/Conn/maxul"), 10l);
//	long maxseeds = pConfig->Read(_T("/Conn/maxseeds"), 0l);
	long maxconn = pConfig->Read(_T("/Conn/maxconn"), 120l);
	long maxulspeed = pConfig->Read(_T("/Conn/maxulspeed"), 0l);
	long maxdlspeed = pConfig->Read(_T("/Conn/maxdlspeed"), 0l);
	
	// INFO: ports are applied in sharkTorrentFrame 's constructor, also where libtorrent:session (m_session) is initalized 
//	long port = pConfig->Read(_T("/Conn/port"), 6881l);
//	long port_end = pConfig->Read(_T("/Conn/port_end"), 6889l);
	// INFO: utpport is not in use
	//long utpport = pConfig->Read(_T("/Conn/utpport"), 4444l);
	wxString fdl  = pConfig->Read(_T("/Dirs/fdl"), wxFileName::GetHomeDir()  + _T("/Desktop/sharktorrent/"));
	fdl.Replace(_T("\\"), _T("/"));	

	// so now...

	// max downloads cannot be set
	// max uploads
	m_session->set_max_uploads(maxul);

	// max seeds cannot be set
	// max conn
	m_session->set_max_connections(maxconn);

	// max upload speed
	int _tmpspeed;
	maxulspeed > 0 ? _tmpspeed = maxulspeed*1024 : _tmpspeed = -1;
	m_session->set_upload_rate_limit(_tmpspeed);

	// max download speed
	maxdlspeed > 0 ? _tmpspeed = maxdlspeed*1024 : _tmpspeed = -1;
	m_session->set_download_rate_limit(_tmpspeed);

	// if port(s) are changed we need to restart sharktorrent
}

void sharkTorrentFrame::RemoveSelected()
{	
	for( int _fs = m_mlistctrl->GetFirstSelected() ;_fs>=0; ) {

		
		int f_selectedRowItemData = m_mlistctrl->GetSelectedItemData(_fs);
		
		if(::wxRemoveFile(m_handles[f_selectedRowItemData].GetStr())) {
			std::cout << "file delete ok\n";
		} else {
			std::cout << "file delete failed\n";
			wxLogError(_T("file delete failed : ") + m_handles[f_selectedRowItemData].GetStr());
		}
	
		m_session->remove_torrent(m_handles[f_selectedRowItemData].GetHandle());
		m_handles.erase(f_selectedRowItemData);
		
	//	m_mlistctrl->updateList( m_handles );
		
		_fs = m_mlistctrl->GetNextSelected(_fs);
		
	}

	this->updateLists();
}

void sharkTorrentFrame::PauseSelected()
{
	for( int _fs = m_mlistctrl->GetFirstSelected() ;_fs>=0; ) {
		int f_selectedRowItemData = m_mlistctrl->GetSelectedItemData(_fs);
		m_handles[f_selectedRowItemData].GetHandle().pause();
		_fs = m_mlistctrl->GetNextSelected(_fs);
	}

	this->updateLists();
}		

void sharkTorrentFrame::ResumeSelected()
{
	for( int _fs = m_mlistctrl->GetFirstSelected() ;_fs>=0; ) {
		int f_selectedRowItemData = m_mlistctrl->GetSelectedItemData(_fs);
		m_handles[f_selectedRowItemData].GetHandle().resume();
		_fs = m_mlistctrl->GetNextSelected(_fs);
	}

	this->updateLists();
}

int sharkTorrentFrame::GetSeedersCount()
{
	int _count = 0;
	
	for (stMapHandles::iterator i = m_handles.begin() ;  i != m_handles.end() && m_handles.size()>0; ++i)
	{
		torrent_handle h = (i->second).GetHandle();
		torrent_status s = h.status();
		/*
		static char const* state_str[] =
		{"queued", "checking", "connecting", "downloading metadata"
				, "downloading", "finished", "seeding", "allocating"};
		*/
		//(*l2)[f_coli++] = wxString::FromAscii(state_str[s.state] );
		if( s.state == 6) _count++;
	}
	return _count;
}
int sharkTorrentFrame::GetNotSeedNotPausedCount()
{
	
	int _count = 0;
	
	for (stMapHandles::iterator i = m_handles.begin() ;  i != m_handles.end() && m_handles.size()>0; ++i)
	{
		torrent_handle h = (i->second).GetHandle();
		torrent_status s = h.status();
		/*
		static char const* state_str[] =
		{"queued", "checking", "connecting", "downloading metadata"
		, "downloading", "finished", "seeding", "allocating"};
		*/
		//(*l2)[f_coli++] = wxString::FromAscii(state_str[s.state] );
		//if( s.state == 4) _count++;
		if( !h.is_seed()  && !h.is_paused()) _count++;
	}
	return _count;
}
