#ifndef _SHARKTORRENTFRAME_H_
#define _SHARKTORRENTFRAME_H_

#include "wx/wx.h"
#include "wx/ipc.h"
#include <iostream>
#include <fstream>
#include <iterator>
#include <exception>

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/exception.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/bind.hpp>

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"
#include "libtorrent/session.hpp"
#include "libtorrent/identify_client.hpp"
#include "libtorrent/alert_types.hpp"
#include "libtorrent/ip_filter.hpp"

#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/splitter.h"
#include "wx/log.h"
#include "wx/dnd.h"
#include "wx/dir.h"

#include "wx/fileconf.h"
#include "wx/utils.h"
#include "wx/wfstream.h"
#include "wx/stdpaths.h"

// separate functions that do not belong to any class
#include "helperfunctions.h"

#include "sttaskbaricon.h"
#include "sttorrentslistctrl.h"
#include "sttorrentmetainfogui.h"
#include "stconfigdialog.h"
#include "stconfigdata.h"
#include "stfiledroptarget.h"
#include "stpairstrhandle.h"
#include "sharktorrent.h"

class sharkTorrentFrame : public wxFrame
{
	public:
		sharkTorrentFrame( const wxString& title, const wxPoint& pos, const wxSize& poss );
		void FirstInit();
		void OnQuit( wxCommandEvent& event );
		void OnOpen( wxCommandEvent& event );
		void OnAbout( wxCommandEvent& event );
		void OnClose( wxCloseEvent& event );
		void OnTimer(wxTimerEvent& event);

		void openSettings(wxCommandEvent& event);
		void ApplySettings();
		
		void RemoveSelected();
		void PauseSelected();
		void ResumeSelected();
		
		void GenerateMenusAndStatusBar();
		
		int GetSeedersCount();
		int GetNotSeedNotPausedCount();

		void DoQuit();
		
		void OpenTorrentFiles(wxArrayString was, bool bootup = false);
		void OpenTorrentFile(wxString s);
		
		void updateLists();

	private:
		DECLARE_EVENT_TABLE();
	private:
		libtorrent::session *m_session;
		wxTimer m_timer;
		stMapHandles m_handles;
		stConfigData	m_confdata;
		stTaskBarIcon	*m_tbi;
		wxString m_lastOpenDir;
		int m_hashmapcount;
		wxLongLong m_lastOnTimer;
	private:
		stTorrentsListCtrl		*m_mlistctrl;
		stTorrentMetainfoGui	*m_mig;
		wxSplitterWindow		*m_splitter;	
};

enum
{
	Menu_File_Quit = 100,
	Menu_File_Open_Torrent,
	Menu_Settings_Configure,
	Menu_Help_About,
	MAIN_TIMER_ID
};

class stIPCServer : public wxServer {
	wxConnectionBase * OnAcceptConnection(const wxString& topic)
	{
		std::cout << "this is OnAcceptConnection, topic=" << topic.mb_str(wxConvUTF8) << "\n";
		wxGetApp().frame->OpenTorrentFile(topic);
		return new wxConnection();
	}
};
#endif
